@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -21,9 +19,12 @@ Tests for `limit` module.
 from unittest import mock
 import uuid
 
+from openstack import exceptions as os_exceptions
 from openstack.identity.v3 import endpoint
 from openstack.identity.v3 import limit as klimit
+from openstack.identity.v3 import region
 from openstack.identity.v3 import registered_limit
+from openstack.identity.v3 import service
 from oslo_config import cfg
 from oslo_config import fixture as config_fixture
 from oslotest import base
@@ -39,13 +40,17 @@ CONF = cfg.CONF
 class TestEnforcer(base.BaseTestCase):
 
     def setUp(self):
-        super(TestEnforcer, self).setUp()
+        super().setUp()
         self.deltas = dict()
         self.config_fixture = self.useFixture(config_fixture.Config(CONF))
         self.config_fixture.config(
             group='oslo_limit',
             auth_type='password'
         )
+        self.config_fixture.config(
+            group='oslo_limit',
+            endpoint_id='ENDPOINT_ID'
+        )
         opts.register_opts(CONF)
         self.config_fixture.config(
             group='oslo_limit',
@@ -222,7 +227,13 @@ class TestEnforcer(base.BaseTestCase):
 
 class TestFlatEnforcer(base.BaseTestCase):
     def setUp(self):
-        super(TestFlatEnforcer, self).setUp()
+        super().setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+        self.config_fixture.config(
+            group='oslo_limit',
+            endpoint_id='ENDPOINT_ID'
+        )
+        opts.register_opts(CONF)
         self.mock_conn = mock.MagicMock()
         limit._SDK_CONNECTION = self.mock_conn
 
@@ -259,7 +270,7 @@ class TestFlatEnforcer(base.BaseTestCase):
         enforcer = limit._FlatEnforcer(mock_usage)
         enforcer.enforce(project_id, deltas)
 
-        self.mock_conn.get_endpoint.assert_called_once_with(None)
+        self.mock_conn.get_endpoint.assert_called_once_with('ENDPOINT_ID')
         mock_get_limits.assert_called_once_with(project_id, ["a", "b"])
         mock_usage.assert_called_once_with(project_id, ["a", "b"])
 
@@ -309,7 +320,13 @@ class TestFlatEnforcer(base.BaseTestCase):
 
 class TestEnforcerUtils(base.BaseTestCase):
     def setUp(self):
-        super(TestEnforcerUtils, self).setUp()
+        super().setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+        self.config_fixture.config(
+            group='oslo_limit',
+            endpoint_id='ENDPOINT_ID'
+        )
+        opts.register_opts(CONF)
         self.mock_conn = mock.MagicMock()
         limit._SDK_CONNECTION = self.mock_conn
 
@@ -320,7 +337,245 @@ class TestEnforcerUtils(base.BaseTestCase):
         utils = limit._EnforcerUtils()
 
         self.assertEqual(fake_endpoint, utils._endpoint)
-        self.mock_conn.get_endpoint.assert_called_once_with(None)
+        self.mock_conn.get_endpoint.assert_called_once_with('ENDPOINT_ID')
+        self.mock_conn.services.assert_not_called()
+        self.mock_conn.endpoints.assert_not_called()
+
+    def test_get_endpoint_no_id(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.mock_conn.get_endpoint.side_effect = \
+            os_exceptions.ResourceNotFound
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_not_called()
+        self.mock_conn.endpoints.assert_not_called()
+
+    def test_get_endpoint_missing(self):
+        self.mock_conn.get_endpoint.side_effect = \
+            os_exceptions.ResourceNotFound
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+        self.mock_conn.get_endpoint.assert_called_once_with('ENDPOINT_ID')
+        self.mock_conn.services.assert_not_called()
+        self.mock_conn.endpoints.assert_not_called()
+
+    def test_get_endpoint_lookup_without_service_opts(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_not_called()
+        self.mock_conn.endpoints.assert_not_called()
+
+    def test_get_endpoint_lookup(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        fake_service = service.Service(id='SERVICE_ID')
+        self.mock_conn.services.return_value = [fake_service]
+        fake_endpoint = endpoint.Endpoint()
+        self.mock_conn.endpoints.return_value = [fake_endpoint]
+
+        utils = limit._EnforcerUtils()
+
+        self.assertEqual(fake_endpoint, utils._endpoint)
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.endpoints.assert_called_once_with(
+            service_id='SERVICE_ID', region_id=None, interface='publicURL')
+
+    def test_get_endpoint_lookup_multiple_endpoints(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        fake_service = service.Service(id='SERVICE_ID')
+        self.mock_conn.services.return_value = [fake_service]
+        self.mock_conn.endpoints.return_value = [
+            endpoint.Endpoint(), endpoint.Endpoint()
+        ]
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.endpoints.assert_called_once_with(
+            service_id='SERVICE_ID', region_id=None, interface='publicURL')
+
+    def test_get_endpoint_lookup_endpoint_not_found(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        fake_service = service.Service(id='SERVICE_ID')
+        self.mock_conn.services.return_value = [fake_service]
+        self.mock_conn.endpoints.side_effect = os_exceptions.ResourceNotFound
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.endpoints.assert_called_once_with(
+            service_id='SERVICE_ID', region_id=None, interface='publicURL')
+
+    def test_get_endpoint_lookup_multiple_service(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        self.mock_conn.services.side_effect = [
+            service.Service(id='SERVICE_ID1'),
+            service.Service(id='SERVICE_ID2')
+        ]
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.endpoints.assert_not_called()
+
+    def test_get_endpoint_lookup_service_not_found(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        self.mock_conn.services.side_effect = os_exceptions.ResourceNotFound
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.endpoints.assert_not_called()
+
+    def test_get_endpoint_lookup_with_region(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_region_name='regionOne'
+        )
+        fake_service = service.Service(id='SERVICE_ID')
+        self.mock_conn.services.return_value = [fake_service]
+        fake_endpoint = endpoint.Endpoint()
+        self.mock_conn.endpoints.return_value = [fake_endpoint]
+        fake_region = region.Region(id='REGION_ID')
+        self.mock_conn.regions.return_value = [fake_region]
+
+        utils = limit._EnforcerUtils()
+
+        self.assertEqual(fake_endpoint, utils._endpoint)
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.regions.assert_called_once_with(name='regionOne')
+        self.mock_conn.endpoints.assert_called_once_with(
+            service_id='SERVICE_ID', region_id='REGION_ID',
+            interface='publicURL')
+
+    def test_get_endpoint_lookup_with_region_not_found(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_region_name='regionOne'
+        )
+        fake_service = service.Service(id='SERVICE_ID')
+        self.mock_conn.services.return_value = [fake_service]
+        fake_endpoint = endpoint.Endpoint()
+        self.mock_conn.endpoints.return_value = [fake_endpoint]
+        self.mock_conn.regions.side_effect = os_exceptions.ResourceNotFound
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.regions.assert_called_once_with(name='regionOne')
+        self.mock_conn.endpoints.assert_not_called()
+
+    def test_get_endpoint_lookup_with_mutliple_regions(self):
+        self.config_fixture.config(group='oslo_limit', endpoint_id=None)
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_type='SERVICE_TYPE'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_service_name='SERVICE_NAME'
+        )
+        self.config_fixture.config(
+            group='oslo_limit', endpoint_region_name='regionOne'
+        )
+        fake_service = service.Service(id='SERVICE_ID')
+        self.mock_conn.services.return_value = [fake_service]
+        fake_endpoint = endpoint.Endpoint()
+        self.mock_conn.endpoints.return_value = [fake_endpoint]
+        self.mock_conn.regions.return_value = [
+            region.Region(id='REGION_ID1'), region.Region(id='REGION_ID2')]
+
+        self.assertRaises(
+            ValueError,
+            limit._EnforcerUtils
+        )
+
+        self.mock_conn.get_endpoint.assert_not_called()
+        self.mock_conn.services.assert_called_once_with(type='SERVICE_TYPE',
+                                                        name='SERVICE_NAME')
+        self.mock_conn.regions.assert_called_once_with(name='regionOne')
+        self.mock_conn.endpoints.assert_not_called()
 
     def test_get_registered_limit_empty(self):
         self.mock_conn.registered_limits.return_value = iter([])
@@ -341,9 +596,8 @@ class TestEnforcerUtils(base.BaseTestCase):
         self.assertEqual(foo, reg_limit)
 
     def test_get_registered_limits(self):
-        fake_endpoint = endpoint.Endpoint()
-        fake_endpoint.service_id = "service_id"
-        fake_endpoint.region_id = "region_id"
+        fake_endpoint = endpoint.Endpoint(service_id='service_id',
+                                          region_id='region_id')
         self.mock_conn.get_endpoint.return_value = fake_endpoint
 
         # a and c have limits, b doesn't have one
@@ -368,9 +622,8 @@ class TestEnforcerUtils(base.BaseTestCase):
         self.assertEqual([('a', 1), ('b', 0), ('c', 2)], limits)
 
     def test_get_project_limits(self):
-        fake_endpoint = endpoint.Endpoint()
-        fake_endpoint.service_id = "service_id"
-        fake_endpoint.region_id = "region_id"
+        fake_endpoint = endpoint.Endpoint(service_id='service_id',
+                                          region_id='region_id')
         self.mock_conn.get_endpoint.return_value = fake_endpoint
         project_id = uuid.uuid4().hex
 
